From mboxrd@z Thu Jan  1 00:00:00 1970
Date: Sun, 6 Dec 2020 13:54:39 +0100
From: Solene Rapenne <solene@perso.pw>
To: list+catgirl@causal.agency
Subject: add OpenBSD specifics code for restricted mode
Message-ID: <20201206135439.4f52c307@kongroo.lan>
MIME-Version: 1.0
Content-Type: multipart/mixed; boundary="MP_/==nP/ERv11neSG110S6t=K7"

--MP_/==nP/ERv11neSG110S6t=K7
Content-Type: text/plain; charset=UTF-8
Content-Transfer-Encoding: quoted-printable
Content-Disposition: inline

Hello,

I was looking for a nice IRC client and found catgirl which was
exactly was I looking for, thanks for that! I've prepared a port
for catgirl, waiting to be reviewed by peers [0]. I found about the
-R flag to switch to restricted mode and for me, it's the perfect
opportunity to add some OpenBSD specific syscalls to limit filesystem
access and restrict system calls, see pledge [1] and unveil [2].

In the past, I did a similar work on irssi but as you may read in
this mail thread [3], it was breaking irssi behavior and wasn't
acceptable. Forking irssi was a solution but it was too much work
to keep the fork in sync with the upstream project. catgirl -R
is a good opportunity to add these features without changing
expected behavior.

I have a work in progress patch (attached to this mail) which
work, at the exception of the absence of the configuration or
data directory. The directories must exists before using unveil()
on them.

I am not sure how to do it correctly without adding too much code,
I could add a recursive mkdir once I get the config path from
configPath() and the data path from dataPath().

I need to know:

1) you are willing to accept this kind of OS specific patch, if
not, I'd have to fork and keep it in sync. Given it's a small change
that would be a pity to keep a separate project for it.

2) what do you think would be a proper way to create the
directories at this stage of the program?

Regards
Sol=C3=A8ne


[0] https://marc.info/?l=3Dopenbsd-ports&m=3D160720917817020&w=3D2
[1] https://man.openbsd.org/pledge
[2] https://man.openbsd.org/unveil
[3] https://marc.info/?t=3D155992729400002&r=3D1&w=3D2

--MP_/==nP/ERv11neSG110S6t=K7
Content-Type: text/x-patch
Content-Transfer-Encoding: 7bit
Content-Disposition: attachment; filename=wip.patch

diff --git a/chat.c b/chat.c
index 296c0c4..08848ae 100644
--- a/chat.c
+++ b/chat.c
@@ -122,6 +122,29 @@ static void signalHandler(int signal) {
 	signals[signal] = 1;
 }
 
+#ifdef __OpenBSD__
+void openbsd_security() {
+	const char	*path = "/";
+	const char *dirs;
+
+	/* config directory */
+	dirs = NULL;
+	path = configPath(&dirs, "");
+	if(unveil(path, "r") == -1)
+		err(1, "can't unveil %s", path);
+
+	/* data directory */
+	dirs = NULL;
+	path = dataPath(&dirs, "");
+	if(unveil(path, "wc") == -1)
+		err(1, "can't unveil %s", path);
+
+	/* restrict system calls to the following categories */
+	if(pledge("inet stdio tty cpath wpath", NULL) == -1)
+		err(1, "can't pledge");
+}
+#endif
+
 int main(int argc, char *argv[]) {
 	setlocale(LC_CTYPE, "");
 
@@ -265,6 +288,12 @@ int main(int argc, char *argv[]) {
 		fcntl(execPipe[1], F_SETFD, FD_CLOEXEC);
 	}
 
+#ifdef __OpenBSD__
+	if(self.restricted) {
+		openbsd_security();
+	}
+#endif
+
 	struct pollfd fds[] = {
 		{ .events = POLLIN, .fd = STDIN_FILENO },
 		{ .events = POLLIN, .fd = irc },

--MP_/==nP/ERv11neSG110S6t=K7--

